@@ -1,9 +1,7 @@
-#ifndef __SP_COLOR_H__
-#define __SP_COLOR_H__
+#ifndef SEEN_SP_COLOR_H
+#define SEEN_SP_COLOR_H
 
-/** \file
- * Colors.
- *
+/*
  * Author:
  *   Lauris Kaplinski <lauris@kaplinski.com>
  *   bulia byak <buliabyak@users.sf.net>
@@ -15,8 +13,8 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
-#include <gdk/gdktypes.h>
 #include <string>
+typedef unsigned int guint32; // uint is guaranteed to hold up to 2^32 − 1
 
 /* Useful composition macros */
 
@@ -25,7 +23,7 @@
 #define SP_RGBA32_B_U(v) (((v) >> 8) & 0xff)
 #define SP_RGBA32_A_U(v) ((v) & 0xff)
 #define SP_COLOR_U_TO_F(v) ((v) / 255.0)
-#define SP_COLOR_F_TO_U(v) ((unsigned int) ((v) * 255.9999))
+#define SP_COLOR_F_TO_U(v) ((unsigned int) ((v) * 255. + .5))
 #define SP_RGBA32_R_F(v) SP_COLOR_U_TO_F (SP_RGBA32_R_U (v))
 #define SP_RGBA32_G_F(v) SP_COLOR_U_TO_F (SP_RGBA32_G_U (v))
 #define SP_RGBA32_B_F(v) SP_COLOR_U_TO_F (SP_RGBA32_B_U (v))
@@ -48,13 +46,14 @@ struct SPColor {
     SPColor& operator= (SPColor const& other);
 
     bool operator == ( SPColor const& other ) const;
+    bool operator != ( SPColor const& other ) const { return !(*this == other); };
     bool isClose( SPColor const& other, float epsilon ) const;
 
     void set( float r, float g, float b );
     void set( guint32 value );
 
-    guint32 toRGBA32( gint alpha ) const;
-    guint32 toRGBA32( gdouble alpha ) const;
+    guint32 toRGBA32( int alpha ) const;
+    guint32 toRGBA32( double alpha ) const;
 
     std::string toString() const;
 
@@ -82,5 +81,4 @@ void sp_color_rgb_to_cmyk_floatv (float *cmyk, float r, float g, float b);
 void sp_color_cmyk_to_rgb_floatv (float *rgb, float c, float m, float y, float k);
 
 
-#endif
-
+#endif // SEEN_SP_COLOR_H
