@@ -5,6 +5,8 @@
  * Authors:
  *   Ted Gould <ted@gould.cx>
  *   Ulf Erikson <ulferikson@users.sf.net>
+ *   Jon A. Cruz <jon@joncruz.org>
+ *   Abhishek Sharma
  *
  * Copyright (C) 2004-2006 Authors
  *
@@ -25,10 +27,7 @@
 #include "extension/print.h"
 #include "extension/db.h"
 #include "extension/output.h"
-#include "display/nr-arena.h"
-#include "display/nr-arena-item.h"
-
-#include <libnr/n-art-bpath.h>
+#include "display/drawing.h"
 
 #include "display/curve.h"
 #include "display/canvas-bpath.h"
@@ -38,15 +37,15 @@
 #include "sp-shape.h"
 
 #include "io/sys.h"
+#include "document.h"
 
 namespace Inkscape {
 namespace Extension {
 namespace Internal {
 
-bool
-CairoRendererOutput::check (Inkscape::Extension::Extension * module)
+bool CairoRendererOutput::check(Inkscape::Extension::Extension * /*module*/)
 {
-	return TRUE;
+    return true;
 }
 
 static bool
@@ -55,21 +54,21 @@ png_render_document_to_file(SPDocument *doc, gchar const *filename)
     CairoRenderer *renderer;
     CairoRenderContext *ctx;
 
-    sp_document_ensure_up_to_date(doc);
+    doc->ensureUpToDate();
 
 /* Start */
-    /* Create new arena */
-    SPItem *base = SP_ITEM(sp_document_root(doc));
-    NRArena *arena = NRArena::create();
-    unsigned dkey = sp_item_display_key_new(1);
-    NRArenaItem *root = sp_item_invoke_show(base, arena, dkey, SP_ITEM_SHOW_DISPLAY);
+
+    SPItem *base = doc->getRoot();
+    Inkscape::Drawing drawing;
+    unsigned dkey = SPItem::display_key_new(1);
+    base->invoke_show(drawing, dkey, SP_ITEM_SHOW_DISPLAY);
     
     /* Create renderer and context */
     renderer = new CairoRenderer();
     ctx = renderer->createContext();
 
     /* Render document */
-    bool ret = renderer->setupDocument(ctx, doc);
+    bool ret = renderer->setupDocument(ctx, doc, TRUE, 0., NULL);
     if (ret) {
         renderer->renderItem(ctx, base);
         ctx->saveAsPng(filename);
@@ -77,10 +76,7 @@ png_render_document_to_file(SPDocument *doc, gchar const *filename)
     }
     renderer->destroyContext(ctx);
 
-    /* Release arena */
-    sp_item_invoke_hide(base, dkey);
-    nr_arena_item_unref(root);
-    nr_object_unref((NRObject *) arena);
+    base->invoke_hide(dkey);
 /* end */
     delete renderer;
 
@@ -94,13 +90,11 @@ png_render_document_to_file(SPDocument *doc, gchar const *filename)
 	\param  doc   Document to be saved
     \param  uri   Filename to save to (probably will end in .png)
 */
-void
-CairoRendererOutput::save (Inkscape::Extension::Output *mod, SPDocument *doc, const gchar *uri)
+void CairoRendererOutput::save(Inkscape::Extension::Output * /*mod*/, SPDocument *doc, gchar const *filename)
 {
-    if (!png_render_document_to_file(doc, uri))
+    if (!png_render_document_to_file(doc, filename)) {
         throw Inkscape::Extension::Output::save_failed();
-
-	return;
+    }
 }
 
 /**
@@ -114,7 +108,7 @@ void
 CairoRendererOutput::init (void)
 {
 	Inkscape::Extension::build_from_mem(
-		"<inkscape-extension>\n"
+		"<inkscape-extension xmlns=\"" INKSCAPE_EXTENSION_URI "\">\n"
 			"<name>Cairo PNG Output</name>\n"
 			"<id>org.inkscape.output.png.cairo</id>\n"
 			"<output>\n"
