@@ -5,7 +5,8 @@
  * Authors:
  *   Ted Gould <ted@gould.cx>
  *   Ulf Erikson <ulferikson@users.sf.net>
- *   Adib Taraben <theAdib@yahoo.com>
+ *   Adib Taraben <theAdib@gmail.com>
+ *   Abhishek Sharma
  *
  * Copyright (C) 2004-2006 Authors
  *
@@ -16,6 +17,7 @@
 #define EXTENSION_INTERNAL_CAIRO_PS_OUT_H
 
 #include "extension/implementation/implementation.h"
+#include "sp-item.h"
 
 #ifdef HAVE_CAIRO_PDF
 
@@ -25,6 +27,18 @@ namespace Internal {
 
 class CairoPsOutput : Inkscape::Extension::Implementation::Implementation {
 
+public:
+    bool check(Inkscape::Extension::Extension *module);
+    void save(Inkscape::Extension::Output *mod,
+              SPDocument *doc,
+              gchar const *filename);
+    static void init();
+    bool textToPath(Inkscape::Extension::Print *ext);
+
+};
+
+class CairoEpsOutput : Inkscape::Extension::Implementation::Implementation {
+
 public:
     bool check(Inkscape::Extension::Extension *module);
     void save(Inkscape::Extension::Output *mod,
@@ -50,4 +64,4 @@ public:
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
