@@ -1,5 +1,3 @@
-#define EXTENSION_INTERNAL_LATEX_TEXT_RENDERER_CPP
-
 /** \file
  * Rendering LaTeX file (pdf/eps/ps+latex output)
  *
@@ -9,6 +7,8 @@
  * Authors:
  *   Johan Engelen <goejendaagh@zonnet.nl>
  *   Miklos Erdelyi <erdelyim@gmail.com>
+ *   Jon A. Cruz <jon@joncruz.org>
+ *   Abhishek Sharma
  *
  * Copyright (C) 2006-2011 Authors
  *
@@ -39,11 +39,12 @@
 #include "sp-rect.h"
 #include "text-editing.h"
 
-#include <unit-constants.h>
+#include "util/units.h"
 
 #include "extension/system.h"
 
 #include "io/sys.h"
+#include "document.h"
 
 namespace Inkscape {
 namespace Extension {
@@ -55,22 +56,23 @@ namespace Internal {
  */
 bool
 latex_render_document_text_to_file( SPDocument *doc, gchar const *filename,
-                                    const gchar * const exportId, bool exportDrawing, bool exportCanvas,
+                                    const gchar * const exportId, bool exportDrawing, bool exportCanvas, float bleedmargin_px,
                                     bool pdflatex)
 {
-    sp_document_ensure_up_to_date(doc);
+    doc->ensureUpToDate();
 
     SPItem *base = NULL;
 
     bool pageBoundingBox = true;
     if (exportId && strcmp(exportId, "")) {
         // we want to export the given item only
-        base = SP_ITEM(doc->getObjectById(exportId));
+        base = dynamic_cast<SPItem *>(doc->getObjectById(exportId));
+        g_assert(base != NULL);
         pageBoundingBox = exportCanvas;
     }
     else {
         // we want to export the entire document from root
-        base = SP_ITEM(sp_document_root(doc));
+        base = doc->getRoot();
         pageBoundingBox = !exportDrawing;
     }
 
@@ -83,7 +85,7 @@ latex_render_document_text_to_file( SPDocument *doc, gchar const *filename,
     bool ret = renderer->setTargetFile(filename);
     if (ret) {
         /* Render document */
-        bool ret = renderer->setupDocument(doc, pageBoundingBox, base);
+        bool ret = renderer->setupDocument(doc, pageBoundingBox, bleedmargin_px, base);
         if (ret) {
             renderer->renderItem(base);
         }
@@ -97,7 +99,9 @@ latex_render_document_text_to_file( SPDocument *doc, gchar const *filename,
 LaTeXTextRenderer::LaTeXTextRenderer(bool pdflatex)
   : _stream(NULL),
     _filename(NULL),
-    _pdflatex(pdflatex)
+    _pdflatex(pdflatex),
+    _omittext_state(EMPTY),
+    _omittext_page(1)
 {
     push_transform(Geom::identity());
 }
@@ -136,20 +140,18 @@ LaTeXTextRenderer::setTargetFile(gchar const *filename) {
         Inkscape::IO::dump_fopen_call(filename_ext, "K");
         FILE *osf = Inkscape::IO::fopen_utf8name(filename_ext, "w+");
         if (!osf) {
-            fprintf(stderr, "inkscape: fopen(%s): %s\n",
-                    filename_ext, strerror(errno));
+            fprintf(stderr, "inkscape: fopen(%s): %s\n", filename_ext, strerror(errno));
+            g_free(filename_ext);
             return false;
         }
         _stream = osf;
         g_free(filename_ext);
     }
 
-    if (_stream) {
-        /* fixme: this is kinda icky */
+    /* fixme: this is kinda icky */
 #if !defined(_WIN32) && !defined(__WIN32__)
-        (void) signal(SIGPIPE, SIG_IGN);
+    (void) signal(SIGPIPE, SIG_IGN);
 #endif
-    }
 
     fprintf(_stream, "%%%% Creator: Inkscape %s, www.inkscape.org\n", PACKAGE_STRING);
     fprintf(_stream, "%%%% PDF/EPS/PS + LaTeX output extension by Johan Engelen, 2010\n");
@@ -207,7 +209,9 @@ static char const preamble[] =
 "    \\errmessage{(Inkscape) Transparency is used (non-zero) for the text in Inkscape, but the package \'transparent.sty\' is not loaded}%\n"
 "    \\renewcommand\\transparent[1]{}%\n"
 "  }%\n"
-"  \\providecommand\\rotatebox[2]{#2}%\n";
+"  \\providecommand\\rotatebox[2]{#2}%\n"
+"  \\newcommand*\\fsize{\\dimexpr\\f@size pt\\relax}%\n"
+"  \\newcommand*\\lineheight[1]{\\fontsize{\\fsize}{#1\\fsize}\\selectfont}%\n";
 
 static char const postamble[] =
 "  \\end{picture}%\n"
@@ -224,35 +228,30 @@ LaTeXTextRenderer::writePostamble()
     fprintf(_stream, "%s", postamble);
 }
 
-void
-LaTeXTextRenderer::sp_group_render(SPItem *item)
+void LaTeXTextRenderer::sp_group_render(SPGroup *group)
 {
-    SPGroup *group = SP_GROUP(item);
-
-    GSList *l = g_slist_reverse(group->childList(false));
-    while (l) {
-        SPObject *o = SP_OBJECT (l->data);
-        if (SP_IS_ITEM(o)) {
-            renderItem (SP_ITEM (o));
+	std::vector<SPObject*> l = (group->childList(false));
+    for(std::vector<SPObject*>::const_iterator x = l.begin(); x != l.end(); ++x){
+        SPItem *item = dynamic_cast<SPItem*>(*x);
+        if (item) {
+            renderItem(item);
         }
-        l = g_slist_remove (l, o);
     }
 }
 
-void
-LaTeXTextRenderer::sp_use_render(SPItem *item)
+void LaTeXTextRenderer::sp_use_render(SPUse *use)
 {
     bool translated = false;
-    SPUse *use = SP_USE(item);
 
     if ((use->x._set && use->x.computed != 0) || (use->y._set && use->y.computed != 0)) {
-        Geom::Matrix tp(Geom::Translate(use->x.computed, use->y.computed));
+        Geom::Affine tp(Geom::Translate(use->x.computed, use->y.computed));
         push_transform(tp);
         translated = true;
     }
 
-    if (use->child && SP_IS_ITEM(use->child)) {
-        renderItem(SP_ITEM(use->child));
+    SPItem *childItem = dynamic_cast<SPItem *>(use->child);
+    if (childItem) {
+        renderItem(childItem);
     }
 
     if (translated) {
@@ -260,31 +259,38 @@ LaTeXTextRenderer::sp_use_render(SPItem *item)
     }
 }
 
-void
-LaTeXTextRenderer::sp_text_render(SPItem *item)
+void LaTeXTextRenderer::sp_text_render(SPText *textobj)
 {
-    SPText *textobj = SP_TEXT (item);
-    SPStyle *style = SP_OBJECT_STYLE (SP_OBJECT(item));
-
-    gchar *str = sp_te_get_string_multiline(item);
-    if (!str) {
+    // Nothing to do here... (so don't emit an empty box)
+    // Also avoids falling out of sync with the CairoRenderer (which won't render anything in this case either)
+    if (textobj->layout.getActualLength() == 0)
         return;
-    }
+
+    // Only PDFLaTeX supports importing a single page of a graphics file,
+    // so only PDF backend gets interleaved text/graphics
+    if (_pdflatex && _omittext_state ==  GRAPHIC_ON_TOP)
+        _omittext_state = NEW_PAGE_ON_GRAPHIC;
+
+    SPStyle *style = textobj->style;
 
     // get position and alignment
     // Align vertically on the baseline of the font (retreived from the anchor point)
     // Align horizontally on anchorpoint
     gchar const *alignment = NULL;
+    gchar const *aligntabular = NULL;
     switch (style->text_anchor.computed) {
     case SP_CSS_TEXT_ANCHOR_START:
-        alignment = "[lb]";
+        alignment = "[lt]";
+        aligntabular = "{l}";
         break;
     case SP_CSS_TEXT_ANCHOR_END:
-        alignment = "[rb]";
+        alignment = "[rt]";
+        aligntabular = "{r}";
         break;
     case SP_CSS_TEXT_ANCHOR_MIDDLE:
     default:
-        alignment = "[b]";
+        alignment = "[t]";
+        aligntabular = "{c}";
         break;
     }
     Geom::Point anchor = textobj->attributes.firstXY() * transform();
@@ -311,11 +317,21 @@ LaTeXTextRenderer::sp_text_render(SPItem *item)
     }
 
     // get rotation
-    Geom::Matrix i2doc = sp_item_i2doc_affine(item);
-    Geom::Matrix wotransl = i2doc.without_translation();
+    Geom::Affine i2doc = textobj->i2doc_affine();
+    Geom::Affine wotransl = i2doc.withoutTranslation();
     double degrees = -180/M_PI * Geom::atan2(wotransl.xAxis());
     bool has_rotation = !Geom::are_near(degrees,0.);
 
+    // get line-height
+    float line_height;
+    if (style->line_height.unit == SP_CSS_UNIT_NONE) {
+        // unitless 'line-height' (use as-is, computed value is relative value)
+        line_height = style->line_height.computed;
+    } else {
+        // 'line-height' with unit (make relative, computed value is absolute value)
+        line_height = style->line_height.computed / style->font_size.computed;
+    }
+
     // write to LaTeX
     Inkscape::SVGOStringStream os;
     os.setf(std::ios::fixed); // don't use scientific notation
@@ -331,45 +347,100 @@ LaTeXTextRenderer::sp_text_render(SPItem *item)
         os << "\\rotatebox{" << degrees << "}{";
     }
     os << "\\makebox(0,0)" << alignment << "{";
-    os << "\\smash{" << str << "}";  // smash the text, to be able to put the makebox coordinates at the baseline
-    if (has_rotation) {
-        os << "}"; // rotatebox end
+    if (line_height != 1) {
+        os << "\\lineheight{" << line_height << "}";
     }
+    os << "\\smash{";
+    os << "\\begin{tabular}[t]" << aligntabular;
+
+        // Walk through all spans in the text object.
+        // Write span strings to LaTeX, associated with font weight and style.
+        Inkscape::Text::Layout const &layout = *(te_get_layout (textobj));
+        for (Inkscape::Text::Layout::iterator li = layout.begin(), le = layout.end();
+             li != le; li.nextStartOfSpan())
+        {
+            Inkscape::Text::Layout::iterator ln = li; 
+            ln.nextStartOfSpan();
+            Glib::ustring uspanstr = sp_te_get_string_multiline (textobj, li, ln);
+            const gchar *spanstr = uspanstr.c_str();
+            if (!spanstr) {
+                continue;
+            }
+
+            bool is_bold = false, is_italic = false, is_oblique = false;
+
+            // newline character only -> don't attempt to add style (will break compilation in LaTeX)
+            if (g_strcmp0(spanstr, "\n")) {
+                SPStyle const &spanstyle = *(sp_te_style_at_position (textobj, li));
+                if (spanstyle.font_weight.computed == SP_CSS_FONT_WEIGHT_500 ||
+                    spanstyle.font_weight.computed == SP_CSS_FONT_WEIGHT_600 ||
+                    spanstyle.font_weight.computed == SP_CSS_FONT_WEIGHT_700 ||
+                    spanstyle.font_weight.computed == SP_CSS_FONT_WEIGHT_800 ||
+                    spanstyle.font_weight.computed == SP_CSS_FONT_WEIGHT_900 ||
+                    spanstyle.font_weight.computed == SP_CSS_FONT_WEIGHT_BOLD ||
+                    spanstyle.font_weight.computed == SP_CSS_FONT_WEIGHT_BOLDER) 
+                {
+                    is_bold = true;
+                    os << "\\textbf{";
+                }
+                if (spanstyle.font_style.computed == SP_CSS_FONT_STYLE_ITALIC) 
+                {
+                    is_italic = true;
+                    os << "\\textit{";
+                }
+                if (spanstyle.font_style.computed == SP_CSS_FONT_STYLE_OBLIQUE) 
+                {
+                    is_oblique = true;
+                    os << "\\textsl{";  // this is an accurate choice if the LaTeX chosen font matches the font in Inkscape. Gives bad results when it is not so...
+                }
+            }
+
+            // replace carriage return with double slash
+            gchar ** splitstr = g_strsplit(spanstr, "\n", 2);
+            os << splitstr[0];
+            if (g_strv_length(splitstr) > 1)
+            {
+                os << "\\\\";
+            }
+            g_strfreev(splitstr);
+
+            if (is_oblique) { os << "}"; } // oblique end
+            if (is_italic) { os << "}"; } // italic end
+            if (is_bold) { os << "}"; } // bold end
+        }
+
+    os << "\\end{tabular}"; // tabular end
+    os << "}"; // smash end
+    if (has_rotation) { os << "}"; } // rotatebox end
     os << "}"; //makebox end
     os << "}%\n"; // put end
 
     fprintf(_stream, "%s", os.str().c_str());
 }
 
-void
-LaTeXTextRenderer::sp_flowtext_render(SPItem * item)
+void LaTeXTextRenderer::sp_flowtext_render(SPFlowtext *flowtext)
 {
 /*
 Flowtext is possible by using a minipage! :)
 Flowing in rectangle is possible, not in arb shape.
 */
 
-    SPFlowtext *flowtext = SP_FLOWTEXT(item);
-    SPStyle *style = SP_OBJECT_STYLE (SP_OBJECT(item));
+    // Only PDFLaTeX supports importing a single page of a graphics file,
+    // so only PDF backend gets interleaved text/graphics
+    if (_pdflatex && _omittext_state ==  GRAPHIC_ON_TOP)
+        _omittext_state = NEW_PAGE_ON_GRAPHIC;
 
-    gchar *strtext = sp_te_get_string_multiline(item);
-    if (!strtext) {
-        return;
-    }
-    // replace carriage return with double slash
-    gchar ** splitstr = g_strsplit(strtext, "\n", -1);
-    gchar *str = g_strjoinv("\\\\ ", splitstr);
-    g_free(strtext);
-    g_strfreev(splitstr);
+    SPStyle *style = flowtext->style;
 
     SPItem *frame_item = flowtext->get_frame(NULL);
-    if (!frame_item || !SP_IS_RECT(frame_item)) {
+    SPRect *frame = dynamic_cast<SPRect *>(frame_item);
+    if (!frame_item || !frame) {
         g_warning("LaTeX export: non-rectangular flowed text shapes are not supported, skipping text.");
         return; // don't know how to handle non-rect frames yet. is quite uncommon for latex users i think
     }
 
-    SPRect *frame = SP_RECT(frame_item);
-    Geom::Rect framebox = sp_rect_get_rect(frame) * transform();
+	// We will transform the coordinates
+    Geom::Rect framebox = frame->getRect();
 
     // get position and alignment
     // Align on topleft corner.
@@ -389,7 +460,10 @@ Flowing in rectangle is possible, not in arb shape.
         // no need to add LaTeX code for standard justified output :)
         break;
     }
-    Geom::Point pos(framebox.corner(3)); //topleft corner
+
+	// The topleft Corner was calculated after rotating the text which results in a wrong Coordinate.
+	// Now, the topleft Corner is rotated after calculating it
+    Geom::Point pos(framebox.corner(0) * transform()); //topleft corner
 
     // determine color and transparency (for now, use rgb color model as it is most native to Inkscape)
     bool has_color = false; // if the item has no color set, don't force black color
@@ -412,8 +486,8 @@ Flowing in rectangle is possible, not in arb shape.
     }
 
     // get rotation
-    Geom::Matrix i2doc = sp_item_i2doc_affine(item);
-    Geom::Matrix wotransl = i2doc.without_translation();
+    Geom::Affine i2doc = flowtext->i2doc_affine();
+    Geom::Affine wotransl = i2doc.withoutTranslation();
     double degrees = -180/M_PI * Geom::atan2(wotransl.xAxis());
     bool has_rotation = !Geom::are_near(degrees,0.);
 
@@ -432,9 +506,61 @@ Flowing in rectangle is possible, not in arb shape.
         os << "\\rotatebox{" << degrees << "}{";
     }
     os << "\\makebox(0,0)" << alignment << "{";
-    os << "\\begin{minipage}{" << framebox.width() << "\\unitlength}";
+
+	// Scale the x width correctly
+    os << "\\begin{minipage}{" << framebox.width() * transform().expansionX() << "\\unitlength}";
     os << justification;
-    os << str;
+
+        // Walk through all spans in the text object.
+        // Write span strings to LaTeX, associated with font weight and style.
+        Inkscape::Text::Layout const &layout = *(te_get_layout(flowtext));
+        for (Inkscape::Text::Layout::iterator li = layout.begin(), le = layout.end(); 
+             li != le; li.nextStartOfSpan())
+        {
+            SPStyle const &spanstyle = *(sp_te_style_at_position(flowtext, li));
+            bool is_bold = false, is_italic = false, is_oblique = false;
+
+            if (spanstyle.font_weight.computed == SP_CSS_FONT_WEIGHT_500 ||
+                spanstyle.font_weight.computed == SP_CSS_FONT_WEIGHT_600 ||
+                spanstyle.font_weight.computed == SP_CSS_FONT_WEIGHT_700 ||
+                spanstyle.font_weight.computed == SP_CSS_FONT_WEIGHT_800 ||
+                spanstyle.font_weight.computed == SP_CSS_FONT_WEIGHT_900 ||
+                spanstyle.font_weight.computed == SP_CSS_FONT_WEIGHT_BOLD ||
+                spanstyle.font_weight.computed == SP_CSS_FONT_WEIGHT_BOLDER) 
+            {
+                is_bold = true;
+                os << "\\textbf{";
+            }
+            if (spanstyle.font_style.computed == SP_CSS_FONT_STYLE_ITALIC) 
+            {
+                is_italic = true;
+                os << "\\textit{";
+            }
+            if (spanstyle.font_style.computed == SP_CSS_FONT_STYLE_OBLIQUE) 
+            {
+                is_oblique = true;
+                os << "\\textsl{";  // this is an accurate choice if the LaTeX chosen font matches the font in Inkscape. Gives bad results when it is not so...
+            }
+
+            Inkscape::Text::Layout::iterator ln = li; 
+            ln.nextStartOfSpan();
+            Glib::ustring uspanstr = sp_te_get_string_multiline(flowtext, li, ln);
+            const gchar *spanstr = uspanstr.c_str();
+            if (!spanstr) {
+                continue;
+            }
+            // replace carriage return with double slash
+            gchar ** splitstr = g_strsplit(spanstr, "\n", -1);
+            gchar *spanstr_new = g_strjoinv("\\\\ ", splitstr);
+            os << spanstr_new;
+            g_strfreev(splitstr);
+            g_free(spanstr_new);
+
+            if (is_oblique) { os << "}"; } // oblique end
+            if (is_italic) { os << "}"; } // italic end
+            if (is_bold) { os << "}"; } // bold end
+        }
+
     os << "\\end{minipage}";
     if (has_rotation) {
         os << "}"; // rotatebox end
@@ -445,13 +571,10 @@ Flowing in rectangle is possible, not in arb shape.
     fprintf(_stream, "%s", os.str().c_str());
 }
 
-void
-LaTeXTextRenderer::sp_root_render(SPItem *item)
+void LaTeXTextRenderer::sp_root_render(SPRoot *root)
 {
-    SPRoot *root = SP_ROOT(item);
-
     push_transform(root->c2p);
-    sp_group_render(item);
+    sp_group_render(root);
     pop_transform();
 }
 
@@ -463,18 +586,37 @@ LaTeXTextRenderer::sp_item_invoke_render(SPItem *item)
         return;
     }
 
-    if (SP_IS_ROOT(item)) {
-        return sp_root_render(item);
-    } else if (SP_IS_GROUP(item)) {
-        return sp_group_render(item);
-    } else if (SP_IS_USE(item)) {
-        sp_use_render(item);
-    } else if (SP_IS_TEXT(item)) {
-        return sp_text_render(item);
-    } else if (SP_IS_FLOWTEXT(item)) {
-        return sp_flowtext_render(item);
+    SPRoot *root = dynamic_cast<SPRoot *>(item);
+    if (root) {
+        sp_root_render(root);
+    } else {
+        SPGroup *group = dynamic_cast<SPGroup *>(item);
+        if (group) {
+            sp_group_render(group);
+        } else {
+            SPUse *use = dynamic_cast<SPUse *>(item);
+            if (use) {
+                sp_use_render(use);
+            } else {
+                SPText *text = dynamic_cast<SPText *>(item);
+                if (text) {
+                    sp_text_render(text);
+                } else {
+                    SPFlowtext *flowtext = dynamic_cast<SPFlowtext *>(item);
+                    if (flowtext) {
+                        sp_flowtext_render(flowtext);
+                    } else {
+                        // Only PDFLaTeX supports importing a single page of a graphics file,
+                        // so only PDF backend gets interleaved text/graphics
+                        if (_pdflatex && (_omittext_state == EMPTY || _omittext_state == NEW_PAGE_ON_GRAPHIC)) {
+                            writeGraphicPage();
+                        }
+                        _omittext_state = GRAPHIC_ON_TOP;
+                    }
+                }
+            }
+        }
     }
-    // We are not interested in writing the other SPItem types to LaTeX
 }
 
 void
@@ -485,48 +627,63 @@ LaTeXTextRenderer::renderItem(SPItem *item)
     pop_transform();
 }
 
+void
+LaTeXTextRenderer::writeGraphicPage(void) {
+    Inkscape::SVGOStringStream os;
+    os.setf(std::ios::fixed); // no scientific notation
+
+    // strip pathname, as it is probably desired. Having a specific path in the TeX file is not convenient.
+    if (_pdflatex)
+        os << "    \\put(0,0){\\includegraphics[width=\\unitlength,page=" << _omittext_page++ << "]{" << _filename << "}}%\n";
+    else
+        os << "    \\put(0,0){\\includegraphics[width=\\unitlength]{" << _filename << "}}%\n";
+
+    fprintf(_stream, "%s", os.str().c_str());
+}
+
 bool
-LaTeXTextRenderer::setupDocument(SPDocument *doc, bool pageBoundingBox, SPItem *base)
+LaTeXTextRenderer::setupDocument(SPDocument *doc, bool pageBoundingBox, float bleedmargin_px, SPItem *base)
 {
 // The boundingbox calculation here should be exactly the same as the one by CairoRenderer::setupDocument !
 
-    if (!base)
-        base = SP_ITEM(sp_document_root(doc));
+    if (!base) {
+        base = doc->getRoot();
+    }
 
-    Geom::OptRect d;
+    Geom::Rect d;
     if (pageBoundingBox) {
-        d = Geom::Rect( Geom::Point(0,0),
-                        Geom::Point(sp_document_width(doc), sp_document_height(doc)) );
+        d = Geom::Rect::from_xywh(Geom::Point(0,0), doc->getDimensions());
     } else {
-        sp_item_invoke_bbox(base, d, sp_item_i2d_affine(base), TRUE, SPItem::RENDERING_BBOX);
-    }
-    if (!d) {
-        g_message("LaTeXTextRenderer: could not retrieve boundingbox.");
-        return false;
+        Geom::OptRect bbox = base->desktopVisualBounds();
+        if (!bbox) {
+            g_message("CairoRenderer: empty bounding box.");
+            return false;
+        }
+        d = *bbox;
     }
+    d.expandBy(bleedmargin_px);
 
     // scale all coordinates, such that the width of the image is 1, this is convenient for scaling the image in LaTeX
-    double scale = 1/(d->width());
-    double _width = d->width() * scale;
-    double _height = d->height() * scale;
+    double scale = 1/(d.width());
+    double _width = d.width() * scale;
+    double _height = d.height() * scale;
     push_transform( Geom::Scale(scale, scale) );
 
     if (!pageBoundingBox)
     {
-        push_transform( Geom::Translate( - d->min() ) );
+        push_transform( Geom::Translate( -d.min() ) );
     }
 
     // flip y-axis
-    push_transform( Geom::Scale(1,-1) * Geom::Translate(0, sp_document_height(doc)) );
+    push_transform( Geom::Scale(1,-1) * Geom::Translate(0, doc->getHeight().value("px")) ); /// @fixme hardcoded desktop transform!
 
     // write the info to LaTeX
     Inkscape::SVGOStringStream os;
     os.setf(std::ios::fixed); // no scientific notation
 
     // scaling of the image when including it in LaTeX
-
     os << "  \\ifx\\svgwidth\\undefined%\n";
-    os << "    \\setlength{\\unitlength}{" << d->width() * PT_PER_PX << "bp}%\n"; // note: 'bp' is the Postscript pt unit in LaTeX, see LP bug #792384
+    os << "    \\setlength{\\unitlength}{" << Inkscape::Util::Quantity::convert(d.width(), "px", "pt") << "bp}%\n"; // note: 'bp' is the Postscript pt unit in LaTeX, see LP bug #792384
     os << "    \\ifx\\svgscale\\undefined%\n";
     os << "      \\relax%\n";
     os << "    \\else%\n";
@@ -540,25 +697,31 @@ LaTeXTextRenderer::setupDocument(SPDocument *doc, bool pageBoundingBox, SPItem *
     os << "  \\makeatother%\n";
 
     os << "  \\begin{picture}(" << _width << "," << _height << ")%\n";
-    // strip pathname, as it is probably desired. Having a specific path in the TeX file is not convenient.
-    os << "    \\put(0,0){\\includegraphics[width=\\unitlength]{" << _filename << "}}%\n";
+
+    // set \baselineskip equal to fontsize (the closest we can seem to get to CSS "line-height: 1;")
+    // and remove column spacing from tabular
+    os << "    \\lineheight{1}%\n";
+    os << "    \\setlength\\tabcolsep{0pt}%\n";
 
     fprintf(_stream, "%s", os.str().c_str());
 
+    if (!_pdflatex)
+        writeGraphicPage();
+
     return true;
 }
 
-Geom::Matrix const &
+Geom::Affine const &
 LaTeXTextRenderer::transform()
 {
     return _transform_stack.top();
 }
 
 void
-LaTeXTextRenderer::push_transform(Geom::Matrix const &tr)
+LaTeXTextRenderer::push_transform(Geom::Affine const &tr)
 {
-    if(_transform_stack.size()){
-        Geom::Matrix tr_top = _transform_stack.top();
+    if(!_transform_stack.empty()){
+        Geom::Affine tr_top = _transform_stack.top();
         _transform_stack.push(tr * tr_top);
     } else {
         _transform_stack.push(tr);
@@ -584,4 +747,4 @@ LaTeXTextRenderer::pop_transform()
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
