@@ -203,7 +203,11 @@ void sp_file_print (Gtk::Window& parentWindow);
  * clean unused defs out of file
  */
 void sp_file_vacuum (SPDocument *doc);
-
+void sp_file_convert_text_baseline_spacing(SPDocument *doc);
+void sp_file_convert_font_name(SPDocument *doc);
+void sp_file_convert_dpi(SPDocument *doc);
+enum File_DPI_Fix { FILE_DPI_UNCHANGED = 0, FILE_DPI_VIEWBOX_SCALED, FILE_DPI_DOCUMENT_SCALED };
+extern int sp_file_convert_dpi_method_commandline;
 
 #endif // SEEN_SP_FILE_H
 
