@@ -69,6 +69,7 @@
 #include "document.h"
 #include "layer-model.h"
 #include "selection.h"
+#include "selection-chemistry.h"
 #include "sp-object.h"
 #include "ui/interface.h"
 #include "print.h"
@@ -351,7 +352,7 @@ struct poptOption options[] = {
 
     {"export-margin", 0,
      POPT_ARG_STRING, &sp_export_margin, SP_ARG_EXPORT_MARGIN,
-     N_("Only for PS/EPS/PDF, sets margin in mm around exported area (default 0)"),
+     N_("Sets margin around exported area (default 0) in units of page size for SVG and mm for PS/EPS/PDF"),
      N_("VALUE")},
 
     {"export-area-snap", 0,
@@ -531,14 +532,14 @@ struct poptOption options[] = {
      NULL},
 
     {"no-convert-text-baseline-spacing", 0,
-    POPT_ARG_NONE, &sp_no_convert_text_baseline_spacing, SP_ARG_NO_CONVERT_TEXT_BASELINE_SPACING,
-    N_("Do not fix legacy (pre-0.92) files' text baseline spacing on opening."),
-    NULL},
+     POPT_ARG_NONE, &sp_no_convert_text_baseline_spacing, SP_ARG_NO_CONVERT_TEXT_BASELINE_SPACING,
+     N_("Do not fix legacy (pre-0.92) files' text baseline spacing on opening."),
+     NULL},
 
     {"convert-dpi-method", 0,
      POPT_ARG_STRING, NULL, SP_ARG_CONVERT_DPI_METHOD,
-     N_("Method used to convert pre-.92 document dpi, if needed."),
-     "[none|scale-viewbox|scale-document]"},
+     N_("Method used to convert pre-.92 document dpi, if needed. ([none|scale-viewbox|scale-document])"),
+     "[...]"},
 
     POPT_AUTOHELP POPT_TABLEEND
 };
@@ -557,6 +558,11 @@ gchar * blankParam = g_strdup("");
  */
 static void _win32_set_inkscape_env(gchar const *exe)
 {
+    // add inkscape directory to DLL search path so dynamically linked extension modules find their libraries
+    wchar_t *exe_w = (wchar_t *)g_utf8_to_utf16(exe, -1, NULL, NULL, NULL);
+    SetDllDirectoryW(exe_w);
+    g_free(exe_w);
+
     gchar const *path = g_getenv("PATH");
     gchar const *pythonpath = g_getenv("PYTHONPATH");
 
@@ -944,7 +950,7 @@ namespace Inkscape {
 namespace UI {
 namespace Tools {
 
-guint get_group0_keyval(GdkEventKey const* event);
+guint get_latin_keyval(GdkEventKey const* event);
 
 }
 }
@@ -1002,7 +1008,7 @@ snooper(GdkEvent *event, gpointer /*data*/) {
             alt_pressed = TRUE && (event->button.state & GDK_MOD1_MASK);
             break;
         case GDK_KEY_PRESS:
-            keyval = Inkscape::UI::Tools::get_group0_keyval(&event->key);
+            keyval = Inkscape::UI::Tools::get_latin_keyval(&event->key);
             if (keyval == GDK_KEY_Alt_L) altL_pressed = TRUE;
             if (keyval == GDK_KEY_Alt_R) altR_pressed = TRUE;
             alt_pressed = alt_pressed || altL_pressed || altR_pressed;
@@ -1013,7 +1019,7 @@ snooper(GdkEvent *event, gpointer /*data*/) {
                 event->key.state &= ~GDK_MOD1_MASK;
             break;
         case GDK_KEY_RELEASE:
-            keyval = Inkscape::UI::Tools::get_group0_keyval(&event->key);
+            keyval = Inkscape::UI::Tools::get_latin_keyval(&event->key);
             if (keyval == GDK_KEY_Alt_L) altL_pressed = FALSE;
             if (keyval == GDK_KEY_Alt_R) altR_pressed = FALSE;
             if (!altL_pressed && !altR_pressed)
@@ -1258,15 +1264,30 @@ static int sp_process_file_list(GSList *fl)
                     sp_item_list_to_curves(items, selected, to_select);
 
                 }
+                if (sp_export_margin) {
+                    gdouble margin = g_ascii_strtod(sp_export_margin, NULL);
+                    doc->ensureUpToDate();
+                    SPNamedView *nv;
+                    Inkscape::XML::Node *nv_repr;
+                    if ((nv = sp_document_namedview(doc, 0)) && (nv_repr = nv->getRepr())) {
+                        sp_repr_set_svg_double(nv_repr, "fit-margin-top", margin);
+                        sp_repr_set_svg_double(nv_repr, "fit-margin-left", margin);
+                        sp_repr_set_svg_double(nv_repr, "fit-margin-right", margin);
+                        sp_repr_set_svg_double(nv_repr, "fit-margin-bottom", margin);
+                    }
+                }
+                if(sp_export_area_drawing) {
+                    fit_canvas_to_drawing(doc, sp_export_margin ? true : false);
+                }
                 if(sp_export_id) {
                     doc->ensureUpToDate();
 
                     // "crop" the document to the specified object, cleaning as we go.
                     SPObject *obj = doc->getObjectById(sp_export_id);
-                    Geom::OptRect const bbox(SP_ITEM(obj)->visualBounds());
+                    Geom::OptRect const bbox(SP_ITEM(obj)->desktopVisualBounds());
 
-                    if (bbox) {
-                        doc->fitToRect(*bbox, false);
+                    if (!sp_export_area_page && bbox) {
+                        doc->fitToRect(*bbox, sp_export_margin ? true : false);
                     }
 
                     if (sp_export_id_only) {
