@@ -3,9 +3,7 @@
 #define SEEN_MOD_360_TEST_H
 
 #include <cxxtest/TestSuite.h>
-
-#include "2geom/isnan.h"
-
+#include <2geom/math-utils.h>
 #include "mod360.h"
 
 
@@ -54,5 +52,5 @@ public:
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
 
