@@ -1,11 +1,12 @@
 
 #include <algorithm>
+#include <2geom/coord.h>
 
-#include "approx-equal.h"
 #include "remove-last.h"
 #include "sp-guide.h"
 #include "sp-guide-constraint.h"
 #include "sp-item.h"
+#include "sp-item-rm-unsatisfied-cns.h"
 
 using std::vector;
 
@@ -14,18 +15,21 @@ void sp_item_rm_unsatisfied_cns(SPItem &item)
     if (item.constraints.empty()) {
         return;
     }
-    SnapPointsWithType snappoints;
-    sp_item_snappoints(&item, false, snappoints, NULL);
+    std::vector<Inkscape::SnapCandidatePoint> snappoints;
+    item.getSnappoints(snappoints, NULL);
     for (unsigned i = item.constraints.size(); i--;) {
         g_assert( i < item.constraints.size() );
         SPGuideConstraint const &cn = item.constraints[i];
         int const snappoint_ix = cn.snappoint_ix;
         g_assert( snappoint_ix < int(snappoints.size()) );
-        if (!approx_equal( sp_guide_distance_from_pt(cn.g, snappoints[snappoint_ix].first), 0) ) {
+
+        if (!Geom::are_near(cn.g->getDistanceFrom(snappoints[snappoint_ix].getPoint()), 0, 1e-2)) {
+
             remove_last(cn.g->attached_items, SPGuideAttachment(&item, cn.snappoint_ix));
+
             g_assert( i < item.constraints.size() );
-            vector<SPGuideConstraint>::iterator const ei(&item.constraints[i]);
-            item.constraints.erase(ei);
+
+            item.constraints.erase(item.constraints.begin() + i);
         }
     }
 }
@@ -40,4 +44,4 @@ void sp_item_rm_unsatisfied_cns(SPItem &item)
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
