@@ -423,7 +423,8 @@ const gchar *ClipboardManagerImpl::getFirstObjectID()
            strcmp(ch->name(), "svg:use") &&
            strcmp(ch->name(), "svg:text") &&
            strcmp(ch->name(), "svg:image") &&
-           strcmp(ch->name(), "svg:rect")
+           strcmp(ch->name(), "svg:rect") &&
+           strcmp(ch->name(), "svg:ellipse")
         ) {
         ch = ch->next();
     }
@@ -655,7 +656,6 @@ Glib::ustring ClipboardManagerImpl::getShapeOrTextObjectId(SPDesktop *desktop)
     return svgd;
 }
 
-
 /**
  * Iterate over a list of items and copy them to the clipboard.
  */
@@ -700,24 +700,6 @@ void ClipboardManagerImpl::_copySelection(Inkscape::Selection *selection)
                 obj_copy = _copyNode(obj, _doc, _root);
             else
                 obj_copy = _copyNode(obj, _doc, _clipnode);
-            SPLPEItem *lpeitem = dynamic_cast<SPLPEItem *>(item);
-            if (lpeitem) {
-                Inkscape::SVGOStringStream os;
-                if (lpeitem->hasPathEffect()) {
-                    for (PathEffectList::iterator it = lpeitem->path_effect_list->begin(); it != lpeitem->path_effect_list->end(); ++it)
-                    {
-                        LivePathEffectObject *lpeobj = (*it)->lpeobject;
-                        if (lpeobj) {
-                            Inkscape::XML::Node * lpeobjcopy = _copyNode(lpeobj->getRepr(), _doc, _defs);
-                            gchar *new_conflict_id = sp_object_get_unique_id(lpeobj, lpeobj->getAttribute("id"));
-                            lpeobjcopy->setAttribute("id", new_conflict_id);
-                            g_free(new_conflict_id);
-                            os << "#" << lpeobjcopy->attribute("id") << ";";
-                        }
-                    }
-                }
-                obj_copy->setAttribute("inkscape:path-effect", os.str().c_str());
-            }
 
             // copy complete inherited style
             SPCSSAttr *css = sp_repr_css_attr_inherited(obj, "style");
@@ -750,6 +732,13 @@ void ClipboardManagerImpl::_copySelection(Inkscape::Selection *selection)
             sp_repr_css_set(_clipnode, style, "style");
             sp_repr_css_attr_unref(style);
         }
+        // copy path effect from the first path
+        if (object) {
+            gchar const *effect =object->getRepr()->attribute("inkscape:path-effect");
+            if (effect) {
+                _clipnode->setAttribute("inkscape:path-effect", effect);
+            }
+        }
     }
 
     Geom::OptRect size = selection->visualBounds();
@@ -852,6 +841,19 @@ void ClipboardManagerImpl::_copyUsedDefs(SPItem *item)
         }
     }
 
+    // For lpe items, copy lpe stack if applicable
+    SPLPEItem *lpeitem = dynamic_cast<SPLPEItem *>(item);
+    if (lpeitem) {
+        if (lpeitem->hasPathEffect()) {
+            for (PathEffectList::iterator it = lpeitem->path_effect_list->begin(); it != lpeitem->path_effect_list->end(); ++it){
+                LivePathEffectObject *lpeobj = (*it)->lpeobject;
+                if (lpeobj) {
+                  _copyNode(lpeobj->getRepr(), _doc, _defs);
+                }
+            }
+        }
+    }
+
     // recurse
     for (SPObject *o = item->children ; o != NULL ; o = o->next) {
         SPItem *childItem = dynamic_cast<SPItem *>(o);
