@@ -5,9 +5,8 @@
 #ifndef SEEN_XML_SP_REPR_ATTR_H
 #define SEEN_XML_SP_REPR_ATTR_H
 
-#include <glib/gquark.h>
-#include <glib/gtypes.h>
-#include "gc-managed.h"
+#include <glib.h>
+#include "inkgc/gc-managed.h"
 #include "util/share.h"
 
 #define SP_REPR_ATTRIBUTE_KEY(a) g_quark_to_string((a)->key)
