@@ -5,17 +5,34 @@ namespace XML {
 
 // Very simple observer that just emits a signal if anything happens to a node
 SignalObserver::SignalObserver()
-    : _oldsel(0)
+    : _oldsel(NULL)
 {}
 
+SignalObserver::~SignalObserver()
+{
+    set(NULL); // if _oldsel!=nullptr, remove observer and decrease refcount
+}
+
 // Add this observer to the SPObject and remove it from any previous object
 void SignalObserver::set(SPObject* o)
 {
-    if(_oldsel && _oldsel->repr)
-        _oldsel->repr->removeObserver(*this);
-    if(o && o->repr)
-        o->repr->addObserver(*this);
-    _oldsel = o;
+  // XML Tree being used direcly in this function in the following code
+  //   while it shouldn't be
+  // Pointer to object is stored, so refcounting should be increased/decreased
+    if(_oldsel) {
+        if (_oldsel->getRepr()) {
+            _oldsel->getRepr()->removeObserver(*this);
+        }
+        sp_object_unref(_oldsel);
+        _oldsel = NULL;
+    }
+    if(o) {
+        if (o->getRepr()) {
+            o->getRepr()->addObserver(*this);
+            sp_object_ref(o);
+            _oldsel = o;
+        }
+    }
 }
 
 void SignalObserver::notifyChildAdded(XML::Node&, XML::Node&, XML::Node*)
